@@ -43,6 +43,10 @@ class Person(reloadable.Reloadable):
 	self.common = ircserv.Hashtable()
 	self.nick = nick
 	self.addr = addr
+        self.post_reload()
+
+    def post_reload(self):
+        self.ensure_defaults([('awaymsg', None)])
 
 reloadable.register_for_reload(Person)
 
