@@ -60,6 +60,12 @@ class Hashtable(reloadable.Reloadable):
     def __delitem__(self, key):
         del self.hash[key_xform(key)]
 
+    def get(self, key, fallback = None):
+        if self.has_key(key):
+            return self[key]
+        else:
+            return fallback
+
     def has_key(self, key):
         return self.hash.has_key(key_xform(key))
 
@@ -93,7 +99,8 @@ class AutoopTimer(reloadable.Reloadable, runloop.SingleTimer):
 	channels = self.serv.channels
 	if channels.has_key(self.chan):
 	    chan = channels[self.chan]
-	    if chan.has_key(self.person) and '@' not in chan[self.person].mode:
+	    if chan.has_key(self.person) and '@' not in chan[self.person].mode \
+	       and '@' in chan[self.serv.people[self.serv.nick]].mode:
 		self.serv.out_line('MODE ' + self.chan + ' +o '
 				   + self.person.nick)
 
@@ -126,7 +133,8 @@ class IRCServ(reloadable.Reloadable, runloop.DataSocket):
 			      ('ping_time', 0.0),
 			      ('quitting', 0),
 			      ('active', 0),
-			      ('initializing', 1)])
+			      ('initializing', 1),
+                              ('awaymsgs', {})])
 
     def init_connect(self, host):
 	self.addr_numeric = host
@@ -247,8 +255,22 @@ class IRCServ(reloadable.Reloadable, runloop.DataSocket):
         event.status(net = self.net, message = args[1])
 
     def handle_301(self, nick, addr, args):
-        event.status(net = self.net, message = args[1] + ' is away: '
-                     + args[2])
+        ## Handling of away message display is maximally complicated, as we
+        ## want to properly handle the people whose nick changes we
+        ## get and somehow handle the others.
+        display = 0
+        person = self.people.get(args[1])
+        if person:
+            if person.awaymsg <> args[2]:
+                display = 1
+                person.awaymsg = args[2]
+        else:
+            if self.awaymsgs.get(args[1]) <> args[2]:
+                display = 1
+                self.awaymsgs[args[1]] = args[2]
+        if display:
+            event.status(net = self.net, message = args[1] + ' is away: '
+                         + args[2])
 
     def handle_311(self, nick, addr, args):
         event.status(net = self.net, message = args[1] + ' (' + args[2] + '@'
@@ -458,11 +480,11 @@ class IRCServ(reloadable.Reloadable, runloop.DataSocket):
 		    self.net.last_msg_from = nick
                     i = 0
                     while i < len(self.net.msg_nicklist):
-                        if low_equal(args[0], self.net.msg_nicklist[i]):
+                        if low_equal(nick, self.net.msg_nicklist[i]):
                             del self.net.msg_nicklist[i]
                             break
                         i = i + 1
-                    self.net.msg_nicklist.insert(0, args[0])
+                    self.net.msg_nicklist.insert(0, nick)
             else:
                 event.decrypt_fail(net = self.net, nick = nick, addr = addr,
                                    target = args[0], message = args[1],
