@@ -145,7 +145,7 @@ def keycmd_select_active_window(event):
 def keycmd_walk_nicklist(event):
     nicks = event.window.buf.net.msg_nicklist
     if nicks:
-	event.editor.set_string('/msg ' + nicks[0])
+	event.editor.set_string('/msg %s ' % nicks[0])
 	event.window.buf.net.msg_nicklist = nicks[1:] + [nicks[0]]
 
 def keycmd_resize(event):
@@ -334,7 +334,9 @@ def cmd_exit(event):
     """exit <reason>
 Quits the connection to the current server and deletes the associated
 windows."""
-    event.window.buf.net.server.out_line('QUIT :' + event.tail)
+    net = event.window.buf.net
+    if net.server <> None:
+        net.server.out_line('QUIT :' + event.tail)
 
 def cmd_help(ev):
     """help [<command>]
@@ -350,7 +352,9 @@ if none is given."""
     else:
         event.help_text(text = 'help <command> for help on any command.  The commands defined are:')
         cmds = ''
-        for k, v in globals().items():
+        glist = globals().items()
+        glist.sort()
+        for k, v in glist:
             if len(k) >= 4 and k[:4] == 'cmd_' and hasattr(v, '__doc__') \
                and v.__doc__:
                 cmds = cmds + k[4:] + ' '
@@ -390,11 +394,17 @@ Leaves the current channel and deletes the associated window."""
     buf = window.buf
     net = buf.net
     server = net.server
-    if ircserv.is_channel(name) and server.channels.has_key(name):
-        server.out_line('PART ' + name + ' :' + event.tail)
+    if main.buffers[net] == buf:
+        if server <> None:
+            server.terminate()
+        net.delete_buffers()
     else:
-        buf.pre_delete()
-        del main.buffers[(net, name)]
+        if ircserv.is_channel(name) and server \
+           and server.channels.has_key(name):
+            server.out_line('PART ' + name + ' :' + event.tail)
+        else:
+            buf.pre_delete()
+            del main.buffers[(net, name)]
 
 def cmd_me(ev):
     """me <message>
@@ -473,6 +483,24 @@ Sends an IRC notice to <channel/nick> containing the text <data>."""
         net.server.out_line('NOTICE ' + chan + ' :' + data)
         event.own_notice(net = net, target = chan, message = data)
 
+def cmd_opall(ev):
+    """opall <nick>*
+Gives operator status to all on the current channel except for the
+explicitly listed ones."""
+    no_ops = string.split(ev.tail, ' ')
+    net = ev.window.buf.net
+    channel_name = ev.window.name
+    channel = net.server.channels[channel_name]
+    nicks = map(lambda v: v[0].nick,
+	filter(lambda v, no_ops=no_ops: '@' not in v[1].mode \
+		and v[0].nick not in no_ops,
+	    channel.hash.values()))
+    while nicks:
+	n = min(3, len(nicks))
+	net.server.out_line('MODE ' + channel_name + ' +' + (n * 'o') + ' '
+	    + reduce(lambda x, y: x + ' ' + y, nicks[:n]))
+	nicks = nicks[n:]
+
 def cmd_ping(event):
     """ping
 Pings the current server."""
